#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wavefile.h"

//,  
//wave -  , nSamples -  , sampleRate -  
//baseFreq -    
void ApplyFilter(short *wave, unsigned int nSamples, unsigned int sampleRate, 
				 unsigned int baseFreq, float timeFilterConst)
{
	// stepFilter    
	//    
	int stepFilter = (sampleRate+0.0f)/baseFreq;
	
	//    baseSignal
	short *baseSignal = (short *) malloc(stepFilter * sizeof(short));
	//  baseSignal 
	memset(baseSignal, 0, stepFilter * sizeof(short));

	// ,        
	//  currStep
	int currStep = stepFilter;
	int j = 0;
	for (int i = 0; i < nSamples; i+= stepFilter)
	{
		//  ,     stepFilter
		if (stepFilter + i > nSamples) 
			//  ""    
			currStep = nSamples - i;
		for (j = 0; j < currStep; j++)
		{
			wave[i + j] -= baseSignal[j];
			baseSignal[j] += (wave[i+j] + 0.0f) / timeFilterConst;
		}
	}
	free(baseSignal);
}


int main(int argc, char *argv[])
{
	if (argc != 4)
	{
		printf("\nUSAGE: %s input_file base_freq time_filter_const\n", argv[0]);
		return 0;
	}
	WAVEPCMFILE wf;
	if (!WaveFileRead(argv[1], &wf))
	{
		printf("ERROR: Unsupported file format.\nSupported format is PCM signed 16 bit mono");
		return 1;
	}
	printf("Processing...\n");
	int baseFreq = 0;
	float timeFilterConst = 2.0;
	sscanf(argv[2], "%d", &baseFreq);
	sscanf(argv[3], "%f", &timeFilterConst);

	if (!baseFreq) 
	{
		printf("\nbase_freq can not be zero\n");
		return 1;
	}
	if (timeFilterConst == 0.0) 
	{
		printf("\ntime_filter_const can not be zero\n");
		return 1;
	}

	ApplyFilter(wf.wave, wf.nSamples, wf.sampleRate, baseFreq, timeFilterConst);

	if (WaveFileWrite(&wf, "out.wav"))
		printf("Done.\nOutput wave saved to file \"out.wav\".");
	else
		printf("ERROR: Unable to create file \"out.wav\".");
	free(wf.wave);
	return 0;
}