/*
     wav 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#pragma pack(1)

struct WAVEPCMFILE
{
	unsigned int sampleRate;
	unsigned int nSamples;
	short *wave;
};

struct WAVEPCM_HEADER
{
	int chunkID;
	unsigned int chunkSize;
	int format;

	int subchunk1ID;
	int subchunk1Size;
	unsigned short audioFormat;
	unsigned short numChannels;
	unsigned int sampleRate;
	unsigned int byteRate;
	unsigned short blockAlign;
	unsigned short bitsPerSample;

	int subchunk2ID;
	unsigned int subchunk2Size;
};

int WaveFileWrite(WAVEPCMFILE *wf, char *dest)
{
	if (!wf || !wf->wave || !dest) return 0;
	WAVEPCM_HEADER head;
	memset(&head, 0, sizeof(WAVEPCM_HEADER));
	head.chunkID = 0x46464952; //"RIFF"
	head.chunkSize = (wf->nSamples << 1) + 36;
	head.format = 0x45564157; //"WAVE"

	head.subchunk1ID = 0x20746d66; //"fmt "
	head.subchunk1Size = 16;
	head.audioFormat = 1;
	head.numChannels = 1;
	head.sampleRate = wf->sampleRate;
	head.byteRate = head.sampleRate*2;
	head.blockAlign = 2;
	head.bitsPerSample = 16;

	head.subchunk2ID = 0x61746164; //"data"
	head.subchunk2Size = wf->nSamples*2;
	FILE *fp = fopen(dest, "wb");
	if (!fp) return 0;
	fwrite(&head, sizeof(WAVEPCM_HEADER), 1, fp);
	fwrite(wf->wave, 2, wf->nSamples, fp);
	fclose(fp);
	return 1;
}

int WaveFileRead(char *src, WAVEPCMFILE *wf)
{
	if (!wf || !wf->wave || !src) return 0;
	WAVEPCM_HEADER head;
	FILE *fp = fopen(src, "rb");
	if (!fp) return 0;
	if (!fread(&head, sizeof(WAVEPCM_HEADER), 1, fp))
	{ 
		fclose(fp);
		return 0;
	}
	if (head.chunkID != 0x46464952 || head.subchunk1ID != 0x20746d66 || 
		head.subchunk2ID != 0x61746164 || head.bitsPerSample != 16 ||
		head.numChannels != 1)
	{ 
		fclose(fp);
		return 0;
	}
	wf->wave = (short *)malloc(head.subchunk2Size);
	if (!wf->wave) return 0;
	wf->nSamples = head.subchunk2Size / 2;
	wf->sampleRate = head.sampleRate;

	fread(wf->wave, 1, head.subchunk2Size, fp);
	fclose(fp);

	return 1;
}